# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, overload

import httpx

from ..types import (
    virtual_account_list_params,
    virtual_account_create_params,
    virtual_account_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.virtual_account import VirtualAccount
from ..types.commission_details_param import CommissionDetailsParam
from ..types.virtual_account_list_response import VirtualAccountListResponse
from ..types.virtual_account_create_response import VirtualAccountCreateResponse
from ..types.virtual_account_update_response import VirtualAccountUpdateResponse

__all__ = ["VirtualAccountResource", "AsyncVirtualAccountResource"]


class VirtualAccountResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VirtualAccountResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return VirtualAccountResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VirtualAccountResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return VirtualAccountResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        commission_details: CommissionDetailsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountCreateResponse:
        """
        Create a virtual account

        Args:
          account_id: The account ID the virtual account should be created under.

          name: The name of the virtual account

          commission_details: Specifies how much of the virtual account's incoming funds should be diverted to
              the primary account

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/virtual-account",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "name": name,
                    "commission_details": commission_details,
                },
                virtual_account_create_params.VirtualAccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualAccountCreateResponse,
        )

    def retrieve(
        self,
        virtual_account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccount:
        """
        Retrieve a single virtual account by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not virtual_account_id:
            raise ValueError(f"Expected a non-empty value for `virtual_account_id` but received {virtual_account_id!r}")
        return self._get(
            f"/virtual-account/{virtual_account_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualAccount,
        )

    @overload
    def update(
        self,
        virtual_account_id: str,
        *,
        action: Literal["update"],
        commission_details: CommissionDetailsParam | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountUpdateResponse:
        """Update an existing virtual account

        Args:
          action: The type of action to take.

        An action of type "update" will update the virtual
              account with the specified properties.

          name: The name to update the virtual account with.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        virtual_account_id: str,
        *,
        action: Literal["close"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountUpdateResponse:
        """Update an existing virtual account

        Args:
          action: The type of action to take.

        An action of type "close" will close the virtual
              account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["action"])
    def update(
        self,
        virtual_account_id: str,
        *,
        action: Literal["update"] | Literal["close"],
        commission_details: CommissionDetailsParam | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountUpdateResponse:
        if not virtual_account_id:
            raise ValueError(f"Expected a non-empty value for `virtual_account_id` but received {virtual_account_id!r}")
        return self._patch(
            f"/virtual-account/{virtual_account_id}",
            body=maybe_transform(
                {
                    "action": action,
                    "commission_details": commission_details,
                    "name": name,
                },
                virtual_account_update_params.VirtualAccountUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualAccountUpdateResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        filter_account_id: str | Omit = omit,
        filter_include_closed_accounts: Literal | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountListResponse:
        """Retrieve a list of virtual accounts under an account.

        The virtual accounts are
        returned in an array, with each item representing a virtual account under that
        account.

        The primary account will also be returned as a virtual account, with an account
        type of 'primary' while actual subaccounts under the primary account will have
        an account type of 'default'.

        Args:
          cursor: A cursor string to fetch the next page of results

          filter_account_id: Pass in an account ID to filter virtual accounts by account ID. This will return
              all virtual accounts that match the account ID passed in.

          filter_include_closed_accounts: Include virtual accounts that have been closed in the query results. By default,
              they will not be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/virtual-account",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "filter_account_id": filter_account_id,
                        "filter_include_closed_accounts": filter_include_closed_accounts,
                    },
                    virtual_account_list_params.VirtualAccountListParams,
                ),
            ),
            cast_to=VirtualAccountListResponse,
        )


class AsyncVirtualAccountResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVirtualAccountResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVirtualAccountResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVirtualAccountResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncVirtualAccountResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        commission_details: CommissionDetailsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountCreateResponse:
        """
        Create a virtual account

        Args:
          account_id: The account ID the virtual account should be created under.

          name: The name of the virtual account

          commission_details: Specifies how much of the virtual account's incoming funds should be diverted to
              the primary account

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/virtual-account",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "name": name,
                    "commission_details": commission_details,
                },
                virtual_account_create_params.VirtualAccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualAccountCreateResponse,
        )

    async def retrieve(
        self,
        virtual_account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccount:
        """
        Retrieve a single virtual account by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not virtual_account_id:
            raise ValueError(f"Expected a non-empty value for `virtual_account_id` but received {virtual_account_id!r}")
        return await self._get(
            f"/virtual-account/{virtual_account_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualAccount,
        )

    @overload
    async def update(
        self,
        virtual_account_id: str,
        *,
        action: Literal["update"],
        commission_details: CommissionDetailsParam | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountUpdateResponse:
        """Update an existing virtual account

        Args:
          action: The type of action to take.

        An action of type "update" will update the virtual
              account with the specified properties.

          name: The name to update the virtual account with.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        virtual_account_id: str,
        *,
        action: Literal["close"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountUpdateResponse:
        """Update an existing virtual account

        Args:
          action: The type of action to take.

        An action of type "close" will close the virtual
              account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["action"])
    async def update(
        self,
        virtual_account_id: str,
        *,
        action: Literal["update"] | Literal["close"],
        commission_details: CommissionDetailsParam | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountUpdateResponse:
        if not virtual_account_id:
            raise ValueError(f"Expected a non-empty value for `virtual_account_id` but received {virtual_account_id!r}")
        return await self._patch(
            f"/virtual-account/{virtual_account_id}",
            body=await async_maybe_transform(
                {
                    "action": action,
                    "commission_details": commission_details,
                    "name": name,
                },
                virtual_account_update_params.VirtualAccountUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualAccountUpdateResponse,
        )

    async def list(
        self,
        *,
        cursor: str | Omit = omit,
        filter_account_id: str | Omit = omit,
        filter_include_closed_accounts: Literal | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualAccountListResponse:
        """Retrieve a list of virtual accounts under an account.

        The virtual accounts are
        returned in an array, with each item representing a virtual account under that
        account.

        The primary account will also be returned as a virtual account, with an account
        type of 'primary' while actual subaccounts under the primary account will have
        an account type of 'default'.

        Args:
          cursor: A cursor string to fetch the next page of results

          filter_account_id: Pass in an account ID to filter virtual accounts by account ID. This will return
              all virtual accounts that match the account ID passed in.

          filter_include_closed_accounts: Include virtual accounts that have been closed in the query results. By default,
              they will not be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/virtual-account",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "filter_account_id": filter_account_id,
                        "filter_include_closed_accounts": filter_include_closed_accounts,
                    },
                    virtual_account_list_params.VirtualAccountListParams,
                ),
            ),
            cast_to=VirtualAccountListResponse,
        )


class VirtualAccountResourceWithRawResponse:
    def __init__(self, virtual_account: VirtualAccountResource) -> None:
        self._virtual_account = virtual_account

        self.create = to_raw_response_wrapper(
            virtual_account.create,
        )
        self.retrieve = to_raw_response_wrapper(
            virtual_account.retrieve,
        )
        self.update = to_raw_response_wrapper(
            virtual_account.update,
        )
        self.list = to_raw_response_wrapper(
            virtual_account.list,
        )


class AsyncVirtualAccountResourceWithRawResponse:
    def __init__(self, virtual_account: AsyncVirtualAccountResource) -> None:
        self._virtual_account = virtual_account

        self.create = async_to_raw_response_wrapper(
            virtual_account.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            virtual_account.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            virtual_account.update,
        )
        self.list = async_to_raw_response_wrapper(
            virtual_account.list,
        )


class VirtualAccountResourceWithStreamingResponse:
    def __init__(self, virtual_account: VirtualAccountResource) -> None:
        self._virtual_account = virtual_account

        self.create = to_streamed_response_wrapper(
            virtual_account.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            virtual_account.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            virtual_account.update,
        )
        self.list = to_streamed_response_wrapper(
            virtual_account.list,
        )


class AsyncVirtualAccountResourceWithStreamingResponse:
    def __init__(self, virtual_account: AsyncVirtualAccountResource) -> None:
        self._virtual_account = virtual_account

        self.create = async_to_streamed_response_wrapper(
            virtual_account.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            virtual_account.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            virtual_account.update,
        )
        self.list = async_to_streamed_response_wrapper(
            virtual_account.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TransactionAggregateResponse"]


class TransactionAggregateResponse(BaseModel):
    count: float
    """The total count of transactions."""

    net_change: float = FieldInfo(alias="netChange")
    """The net change, which is `totalIn` + `totalOut`."""

    total_in: float = FieldInfo(alias="totalIn")
    """The total amount of money that arrived into the account, in cents."""

    total_out: float = FieldInfo(alias="totalOut")
    """The total amount of money that was sent out of the account, in cents."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import card_group_list_params, card_group_create_params, card_group_update_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .spending_constraint import (
    SpendingConstraintResource,
    AsyncSpendingConstraintResource,
    SpendingConstraintResourceWithRawResponse,
    AsyncSpendingConstraintResourceWithRawResponse,
    SpendingConstraintResourceWithStreamingResponse,
    AsyncSpendingConstraintResourceWithStreamingResponse,
)
from ...types.card_group.card_group import CardGroup
from ...types.card_group_utilization import CardGroupUtilization
from ...types.card_group_list_response import CardGroupListResponse
from ...types.card.spending_constraint_param import SpendingConstraintParam

__all__ = ["CardGroupResource", "AsyncCardGroupResource"]


class CardGroupResource(SyncAPIResource):
    @cached_property
    def spending_constraint(self) -> SpendingConstraintResource:
        return SpendingConstraintResource(self._client)

    @cached_property
    def with_raw_response(self) -> CardGroupResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CardGroupResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CardGroupResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return CardGroupResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        spending_constraint: SpendingConstraintParam | Omit = omit,
        virtual_account_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroup:
        """
        Create a card group.

        Args:
          spending_constraint: A constraint that can be applied to a CardGroupSpendingRule

          virtual_account_id: The ID of the virtual account to associate with the card group. If not provided,
              the primary virtual account will be used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/card-group",
            body=maybe_transform(
                {
                    "name": name,
                    "spending_constraint": spending_constraint,
                    "virtual_account_id": virtual_account_id,
                },
                card_group_create_params.CardGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroup,
        )

    def retrieve(
        self,
        card_group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroup:
        """
        Fetch details for a single card group by card group ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_group_id:
            raise ValueError(f"Expected a non-empty value for `card_group_id` but received {card_group_id!r}")
        return self._get(
            f"/card-group/{card_group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroup,
        )

    def update(
        self,
        card_group_id: str,
        *,
        name: str | Omit = omit,
        spending_constraint: Optional[SpendingConstraintParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroup:
        """
        Update a card group

        Args:
          spending_constraint: Explicitly set this value to null to remove all card group level spending
              constraints.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_group_id:
            raise ValueError(f"Expected a non-empty value for `card_group_id` but received {card_group_id!r}")
        return self._patch(
            f"/card-group/{card_group_id}",
            body=maybe_transform(
                {
                    "name": name,
                    "spending_constraint": spending_constraint,
                },
                card_group_update_params.CardGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroup,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        filter_name: str | Omit = omit,
        filter_status: Literal["active", "archived"] | Omit = omit,
        filter_virtual_account_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroupListResponse:
        """
        Retrieve the list of card groups that the user owns.

        Args:
          cursor: A cursor string to fetch the next page of results

          filter_name: Pass in a name to filter for card groups with a matching name.

          filter_status: Filter card groups by status. Defaults to 'active' if not provided.

          filter_virtual_account_id: Pass in a virtual account ID to filter for card groups under a specific virtual
              account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/card-group",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "filter_name": filter_name,
                        "filter_status": filter_status,
                        "filter_virtual_account_id": filter_virtual_account_id,
                    },
                    card_group_list_params.CardGroupListParams,
                ),
            ),
            cast_to=CardGroupListResponse,
        )

    def get_utilization(
        self,
        card_group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroupUtilization:
        """
        Get a card group's current utilization

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_group_id:
            raise ValueError(f"Expected a non-empty value for `card_group_id` but received {card_group_id!r}")
        return self._get(
            f"/card-group/{card_group_id}/utilization",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroupUtilization,
        )


class AsyncCardGroupResource(AsyncAPIResource):
    @cached_property
    def spending_constraint(self) -> AsyncSpendingConstraintResource:
        return AsyncSpendingConstraintResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCardGroupResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCardGroupResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCardGroupResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncCardGroupResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        spending_constraint: SpendingConstraintParam | Omit = omit,
        virtual_account_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroup:
        """
        Create a card group.

        Args:
          spending_constraint: A constraint that can be applied to a CardGroupSpendingRule

          virtual_account_id: The ID of the virtual account to associate with the card group. If not provided,
              the primary virtual account will be used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/card-group",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "spending_constraint": spending_constraint,
                    "virtual_account_id": virtual_account_id,
                },
                card_group_create_params.CardGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroup,
        )

    async def retrieve(
        self,
        card_group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroup:
        """
        Fetch details for a single card group by card group ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_group_id:
            raise ValueError(f"Expected a non-empty value for `card_group_id` but received {card_group_id!r}")
        return await self._get(
            f"/card-group/{card_group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroup,
        )

    async def update(
        self,
        card_group_id: str,
        *,
        name: str | Omit = omit,
        spending_constraint: Optional[SpendingConstraintParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroup:
        """
        Update a card group

        Args:
          spending_constraint: Explicitly set this value to null to remove all card group level spending
              constraints.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_group_id:
            raise ValueError(f"Expected a non-empty value for `card_group_id` but received {card_group_id!r}")
        return await self._patch(
            f"/card-group/{card_group_id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "spending_constraint": spending_constraint,
                },
                card_group_update_params.CardGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroup,
        )

    async def list(
        self,
        *,
        cursor: str | Omit = omit,
        filter_name: str | Omit = omit,
        filter_status: Literal["active", "archived"] | Omit = omit,
        filter_virtual_account_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroupListResponse:
        """
        Retrieve the list of card groups that the user owns.

        Args:
          cursor: A cursor string to fetch the next page of results

          filter_name: Pass in a name to filter for card groups with a matching name.

          filter_status: Filter card groups by status. Defaults to 'active' if not provided.

          filter_virtual_account_id: Pass in a virtual account ID to filter for card groups under a specific virtual
              account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/card-group",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "filter_name": filter_name,
                        "filter_status": filter_status,
                        "filter_virtual_account_id": filter_virtual_account_id,
                    },
                    card_group_list_params.CardGroupListParams,
                ),
            ),
            cast_to=CardGroupListResponse,
        )

    async def get_utilization(
        self,
        card_group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroupUtilization:
        """
        Get a card group's current utilization

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_group_id:
            raise ValueError(f"Expected a non-empty value for `card_group_id` but received {card_group_id!r}")
        return await self._get(
            f"/card-group/{card_group_id}/utilization",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroupUtilization,
        )


class CardGroupResourceWithRawResponse:
    def __init__(self, card_group: CardGroupResource) -> None:
        self._card_group = card_group

        self.create = to_raw_response_wrapper(
            card_group.create,
        )
        self.retrieve = to_raw_response_wrapper(
            card_group.retrieve,
        )
        self.update = to_raw_response_wrapper(
            card_group.update,
        )
        self.list = to_raw_response_wrapper(
            card_group.list,
        )
        self.get_utilization = to_raw_response_wrapper(
            card_group.get_utilization,
        )

    @cached_property
    def spending_constraint(self) -> SpendingConstraintResourceWithRawResponse:
        return SpendingConstraintResourceWithRawResponse(self._card_group.spending_constraint)


class AsyncCardGroupResourceWithRawResponse:
    def __init__(self, card_group: AsyncCardGroupResource) -> None:
        self._card_group = card_group

        self.create = async_to_raw_response_wrapper(
            card_group.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            card_group.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            card_group.update,
        )
        self.list = async_to_raw_response_wrapper(
            card_group.list,
        )
        self.get_utilization = async_to_raw_response_wrapper(
            card_group.get_utilization,
        )

    @cached_property
    def spending_constraint(self) -> AsyncSpendingConstraintResourceWithRawResponse:
        return AsyncSpendingConstraintResourceWithRawResponse(self._card_group.spending_constraint)


class CardGroupResourceWithStreamingResponse:
    def __init__(self, card_group: CardGroupResource) -> None:
        self._card_group = card_group

        self.create = to_streamed_response_wrapper(
            card_group.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            card_group.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            card_group.update,
        )
        self.list = to_streamed_response_wrapper(
            card_group.list,
        )
        self.get_utilization = to_streamed_response_wrapper(
            card_group.get_utilization,
        )

    @cached_property
    def spending_constraint(self) -> SpendingConstraintResourceWithStreamingResponse:
        return SpendingConstraintResourceWithStreamingResponse(self._card_group.spending_constraint)


class AsyncCardGroupResourceWithStreamingResponse:
    def __init__(self, card_group: AsyncCardGroupResource) -> None:
        self._card_group = card_group

        self.create = async_to_streamed_response_wrapper(
            card_group.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            card_group.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            card_group.update,
        )
        self.list = async_to_streamed_response_wrapper(
            card_group.list,
        )
        self.get_utilization = async_to_streamed_response_wrapper(
            card_group.get_utilization,
        )

    @cached_property
    def spending_constraint(self) -> AsyncSpendingConstraintResourceWithStreamingResponse:
        return AsyncSpendingConstraintResourceWithStreamingResponse(self._card_group.spending_constraint)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["StatementListParams"]


class StatementListParams(TypedDict, total=False):
    end_time: Annotated[str, PropertyInfo(alias="endTime")]
    """End time for use in retrieval of statements (ISO 8601)."""

    limit: str
    """The number of items to return (default 100, max 1000)"""

    offset: str
    """The ID of the last item in the previous page of results"""

    start_time: Annotated[str, PropertyInfo(alias="startTime")]
    """Start time for use in retrieval of statements (ISO 8601)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)
from slash_sdk.types.fdx.accounts import StatementListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestStatements:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: SlashSDK) -> None:
        statement = client.fdx.accounts.statements.list(
            account_id="accountId",
        )
        assert_matches_type(StatementListResponse, statement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: SlashSDK) -> None:
        statement = client.fdx.accounts.statements.list(
            account_id="accountId",
            end_time="endTime",
            limit="limit",
            offset="offset",
            start_time="startTime",
        )
        assert_matches_type(StatementListResponse, statement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: SlashSDK) -> None:
        response = client.fdx.accounts.statements.with_raw_response.list(
            account_id="accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement = response.parse()
        assert_matches_type(StatementListResponse, statement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: SlashSDK) -> None:
        with client.fdx.accounts.statements.with_streaming_response.list(
            account_id="accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement = response.parse()
            assert_matches_type(StatementListResponse, statement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.fdx.accounts.statements.with_raw_response.list(
                account_id="",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_pdf(self, client: SlashSDK, respx_mock: MockRouter) -> None:
        respx_mock.get("/fdx/accounts/accountId/statements/statementId").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        statement = client.fdx.accounts.statements.retrieve_pdf(
            statement_id="statementId",
            account_id="accountId",
        )
        assert statement.is_closed
        assert statement.json() == {"foo": "bar"}
        assert cast(Any, statement.is_closed) is True
        assert isinstance(statement, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_retrieve_pdf(self, client: SlashSDK, respx_mock: MockRouter) -> None:
        respx_mock.get("/fdx/accounts/accountId/statements/statementId").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        statement = client.fdx.accounts.statements.with_raw_response.retrieve_pdf(
            statement_id="statementId",
            account_id="accountId",
        )

        assert statement.is_closed is True
        assert statement.http_request.headers.get("X-Stainless-Lang") == "python"
        assert statement.json() == {"foo": "bar"}
        assert isinstance(statement, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_retrieve_pdf(self, client: SlashSDK, respx_mock: MockRouter) -> None:
        respx_mock.get("/fdx/accounts/accountId/statements/statementId").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.fdx.accounts.statements.with_streaming_response.retrieve_pdf(
            statement_id="statementId",
            account_id="accountId",
        ) as statement:
            assert not statement.is_closed
            assert statement.http_request.headers.get("X-Stainless-Lang") == "python"

            assert statement.json() == {"foo": "bar"}
            assert cast(Any, statement.is_closed) is True
            assert isinstance(statement, StreamedBinaryAPIResponse)

        assert cast(Any, statement.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_retrieve_pdf(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.fdx.accounts.statements.with_raw_response.retrieve_pdf(
                statement_id="statementId",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `statement_id` but received ''"):
            client.fdx.accounts.statements.with_raw_response.retrieve_pdf(
                statement_id="",
                account_id="accountId",
            )


class TestAsyncStatements:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncSlashSDK) -> None:
        statement = await async_client.fdx.accounts.statements.list(
            account_id="accountId",
        )
        assert_matches_type(StatementListResponse, statement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        statement = await async_client.fdx.accounts.statements.list(
            account_id="accountId",
            end_time="endTime",
            limit="limit",
            offset="offset",
            start_time="startTime",
        )
        assert_matches_type(StatementListResponse, statement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.fdx.accounts.statements.with_raw_response.list(
            account_id="accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement = await response.parse()
        assert_matches_type(StatementListResponse, statement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.fdx.accounts.statements.with_streaming_response.list(
            account_id="accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement = await response.parse()
            assert_matches_type(StatementListResponse, statement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.fdx.accounts.statements.with_raw_response.list(
                account_id="",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_pdf(self, async_client: AsyncSlashSDK, respx_mock: MockRouter) -> None:
        respx_mock.get("/fdx/accounts/accountId/statements/statementId").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        statement = await async_client.fdx.accounts.statements.retrieve_pdf(
            statement_id="statementId",
            account_id="accountId",
        )
        assert statement.is_closed
        assert await statement.json() == {"foo": "bar"}
        assert cast(Any, statement.is_closed) is True
        assert isinstance(statement, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_retrieve_pdf(self, async_client: AsyncSlashSDK, respx_mock: MockRouter) -> None:
        respx_mock.get("/fdx/accounts/accountId/statements/statementId").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        statement = await async_client.fdx.accounts.statements.with_raw_response.retrieve_pdf(
            statement_id="statementId",
            account_id="accountId",
        )

        assert statement.is_closed is True
        assert statement.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await statement.json() == {"foo": "bar"}
        assert isinstance(statement, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_retrieve_pdf(self, async_client: AsyncSlashSDK, respx_mock: MockRouter) -> None:
        respx_mock.get("/fdx/accounts/accountId/statements/statementId").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.fdx.accounts.statements.with_streaming_response.retrieve_pdf(
            statement_id="statementId",
            account_id="accountId",
        ) as statement:
            assert not statement.is_closed
            assert statement.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await statement.json() == {"foo": "bar"}
            assert cast(Any, statement.is_closed) is True
            assert isinstance(statement, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, statement.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_retrieve_pdf(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.fdx.accounts.statements.with_raw_response.retrieve_pdf(
                statement_id="statementId",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `statement_id` but received ''"):
            await async_client.fdx.accounts.statements.with_raw_response.retrieve_pdf(
                statement_id="",
                account_id="accountId",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["WebhookUpdateParams"]


class WebhookUpdateParams(TypedDict, total=False):
    status: Required[Literal["active", "paused", "archived"]]
    """The desired status for the endpoint:

    - `active`: Enable/re-enable the endpoint
    - `paused`: Pause the endpoint (notifications are queued)
    - `archived`: Archive the endpoint (soft delete)
    """

    reason: str
    """Optional reason for the status change (for audit purposes)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["CardRetrieveParams"]


class CardRetrieveParams(TypedDict, total=False):
    include_cvv: Literal["true", "false"]

    include_pan: Literal["true", "false"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import developer_application_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.developer_application_model import DeveloperApplicationModel
from ..types.developer_application_data_param import DeveloperApplicationDataParam

__all__ = ["DeveloperApplicationResource", "AsyncDeveloperApplicationResource"]


class DeveloperApplicationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DeveloperApplicationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DeveloperApplicationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DeveloperApplicationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return DeveloperApplicationResourceWithStreamingResponse(self)

    def retrieve(
        self,
        developer_application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeveloperApplicationModel:
        """
        Get a DeveloperApplication

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not developer_application_id:
            raise ValueError(
                f"Expected a non-empty value for `developer_application_id` but received {developer_application_id!r}"
            )
        return self._get(
            f"/developer-application/{developer_application_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeveloperApplicationModel,
        )

    def update(
        self,
        developer_application_id: str,
        *,
        data: DeveloperApplicationDataParam | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeveloperApplicationModel:
        """
        Update a DeveloperApplication

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not developer_application_id:
            raise ValueError(
                f"Expected a non-empty value for `developer_application_id` but received {developer_application_id!r}"
            )
        return self._patch(
            f"/developer-application/{developer_application_id}",
            body=maybe_transform(
                {
                    "data": data,
                    "name": name,
                },
                developer_application_update_params.DeveloperApplicationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeveloperApplicationModel,
        )

    def create_or_regenerate_secret(
        self,
        developer_application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeveloperApplicationModel:
        """
        Create or regenerate a DeveloperApplication's OAuthClientSecret

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not developer_application_id:
            raise ValueError(
                f"Expected a non-empty value for `developer_application_id` but received {developer_application_id!r}"
            )
        return self._post(
            f"/developer-application/{developer_application_id}/secret",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeveloperApplicationModel,
        )


class AsyncDeveloperApplicationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDeveloperApplicationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDeveloperApplicationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDeveloperApplicationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncDeveloperApplicationResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        developer_application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeveloperApplicationModel:
        """
        Get a DeveloperApplication

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not developer_application_id:
            raise ValueError(
                f"Expected a non-empty value for `developer_application_id` but received {developer_application_id!r}"
            )
        return await self._get(
            f"/developer-application/{developer_application_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeveloperApplicationModel,
        )

    async def update(
        self,
        developer_application_id: str,
        *,
        data: DeveloperApplicationDataParam | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeveloperApplicationModel:
        """
        Update a DeveloperApplication

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not developer_application_id:
            raise ValueError(
                f"Expected a non-empty value for `developer_application_id` but received {developer_application_id!r}"
            )
        return await self._patch(
            f"/developer-application/{developer_application_id}",
            body=await async_maybe_transform(
                {
                    "data": data,
                    "name": name,
                },
                developer_application_update_params.DeveloperApplicationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeveloperApplicationModel,
        )

    async def create_or_regenerate_secret(
        self,
        developer_application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeveloperApplicationModel:
        """
        Create or regenerate a DeveloperApplication's OAuthClientSecret

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not developer_application_id:
            raise ValueError(
                f"Expected a non-empty value for `developer_application_id` but received {developer_application_id!r}"
            )
        return await self._post(
            f"/developer-application/{developer_application_id}/secret",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeveloperApplicationModel,
        )


class DeveloperApplicationResourceWithRawResponse:
    def __init__(self, developer_application: DeveloperApplicationResource) -> None:
        self._developer_application = developer_application

        self.retrieve = to_raw_response_wrapper(
            developer_application.retrieve,
        )
        self.update = to_raw_response_wrapper(
            developer_application.update,
        )
        self.create_or_regenerate_secret = to_raw_response_wrapper(
            developer_application.create_or_regenerate_secret,
        )


class AsyncDeveloperApplicationResourceWithRawResponse:
    def __init__(self, developer_application: AsyncDeveloperApplicationResource) -> None:
        self._developer_application = developer_application

        self.retrieve = async_to_raw_response_wrapper(
            developer_application.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            developer_application.update,
        )
        self.create_or_regenerate_secret = async_to_raw_response_wrapper(
            developer_application.create_or_regenerate_secret,
        )


class DeveloperApplicationResourceWithStreamingResponse:
    def __init__(self, developer_application: DeveloperApplicationResource) -> None:
        self._developer_application = developer_application

        self.retrieve = to_streamed_response_wrapper(
            developer_application.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            developer_application.update,
        )
        self.create_or_regenerate_secret = to_streamed_response_wrapper(
            developer_application.create_or_regenerate_secret,
        )


class AsyncDeveloperApplicationResourceWithStreamingResponse:
    def __init__(self, developer_application: AsyncDeveloperApplicationResource) -> None:
        self._developer_application = developer_application

        self.retrieve = async_to_streamed_response_wrapper(
            developer_application.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            developer_application.update,
        )
        self.create_or_regenerate_secret = async_to_streamed_response_wrapper(
            developer_application.create_or_regenerate_secret,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["InvestmentAccount", "Currency"]


class Currency(BaseModel):
    currency_code: str = FieldInfo(alias="currencyCode")
    """The ISO 4217 currency code for the currency that the account is denominated in.

    For example, "USD" for US Dollars.
    """


class InvestmentAccount(BaseModel):
    """An investment account. For example, a 401K or IRA."""

    account_id: str = FieldInfo(alias="accountId")
    """Long-term persistent identity of the account, though not an account number."""

    account_type: Literal[
        "401A",
        "401K",
        "403B",
        "529",
        "BROKERAGEPRODUCT",
        "COVERDELL",
        "DIGITALASSET",
        "DEFINEDBENEFIT",
        "ESOP",
        "GUARDIAN",
        "INSTITUTIONALTRUST",
        "IRA",
        "KEOGH",
        "NONQUALIFIEDPLAN",
        "OTHERINVESTMENT",
        "ROLLOVER",
        "ROTH",
        "SARSEP",
        "TAXABLE",
        "TDA",
        "TRUST",
        "TERM",
        "UGMA",
        "UTMA",
    ] = FieldInfo(alias="accountType")

    currency: Currency

    product_name: str = FieldInfo(alias="productName")
    """Marketed product name for this account.

    You can use this in UIs to assist in account selection. For example, "Slash
    Platinum".
    """

    status: Literal[
        "CLOSED", "DELINQUENT", "NEGATIVECURRENTBALANCE", "OPEN", "PAID", "PENDINGCLOSE", "PENDINGOPEN", "RESTRICTED"
    ]

    account_number_display: Optional[str] = FieldInfo(alias="accountNumberDisplay", default=None)
    """
    A masked version of the account number, which includes the last 4 digits in
    plain text.
    """

    nickname: Optional[str] = None
    """A nickname or label for the account.

    This will be unique to the account, even if the user has multiple accounts with
    the same product name. You can use this in UIs to assist in account selection.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TransferCreateVirtualAccountTransferResponse"]


class TransferCreateVirtualAccountTransferResponse(BaseModel):
    transfer_id: str = FieldInfo(alias="transferId")
    """The ID of the transfer that was created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types.card import SpendingConstraint

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSpendingConstraint:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_full(self, client: SlashSDK) -> None:
        spending_constraint = client.card_group.spending_constraint.update_full(
            card_group_id="cardGroupId",
        )
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_full_with_all_params(self, client: SlashSDK) -> None:
        spending_constraint = client.card_group.spending_constraint.update_full(
            card_group_id="cardGroupId",
            country_rule={
                "countries": ["string"],
                "restriction": "allowlist",
            },
            merchant_category_code_rule={
                "merchant_category_codes": ["string"],
                "restriction": "allowlist",
            },
            merchant_category_rule={
                "merchant_categories": ["string"],
                "restriction": "allowlist",
            },
            merchant_rule={
                "merchants": ["string"],
                "restriction": "allowlist",
            },
            spending_rule={
                "transaction_size_limit": {
                    "maximum": {"amount_cents": 0},
                    "minimum": {"amount_cents": 0},
                },
                "utilization_limit": {
                    "limit_amount": {"amount_cents": 0},
                    "preset": "daily",
                    "start_date": "startDate",
                    "timezone": "timezone",
                },
                "utilization_limit_v2": [
                    {
                        "limit_amount": {"amount_cents": 0},
                        "preset": "daily",
                        "start_date": "startDate",
                        "timezone": "timezone",
                    }
                ],
            },
        )
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_full(self, client: SlashSDK) -> None:
        response = client.card_group.spending_constraint.with_raw_response.update_full(
            card_group_id="cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spending_constraint = response.parse()
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_full(self, client: SlashSDK) -> None:
        with client.card_group.spending_constraint.with_streaming_response.update_full(
            card_group_id="cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spending_constraint = response.parse()
            assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_full(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            client.card_group.spending_constraint.with_raw_response.update_full(
                card_group_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_partial(self, client: SlashSDK) -> None:
        spending_constraint = client.card_group.spending_constraint.update_partial(
            card_group_id="cardGroupId",
        )
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_partial_with_all_params(self, client: SlashSDK) -> None:
        spending_constraint = client.card_group.spending_constraint.update_partial(
            card_group_id="cardGroupId",
            country_rule={
                "countries": ["string"],
                "restriction": "allowlist",
            },
            merchant_category_code_rule={
                "merchant_category_codes": ["string"],
                "restriction": "allowlist",
            },
            merchant_category_rule={
                "merchant_categories": ["string"],
                "restriction": "allowlist",
            },
            merchant_rule={
                "merchants": ["string"],
                "restriction": "allowlist",
            },
            spending_rule={
                "transaction_size_limit": {
                    "maximum": {"amount_cents": 0},
                    "minimum": {"amount_cents": 0},
                },
                "utilization_limit": {
                    "limit_amount": {"amount_cents": 0},
                    "preset": "daily",
                    "start_date": "startDate",
                    "timezone": "timezone",
                },
                "utilization_limit_v2": [
                    {
                        "limit_amount": {"amount_cents": 0},
                        "preset": "daily",
                        "start_date": "startDate",
                        "timezone": "timezone",
                    }
                ],
            },
        )
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_partial(self, client: SlashSDK) -> None:
        response = client.card_group.spending_constraint.with_raw_response.update_partial(
            card_group_id="cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spending_constraint = response.parse()
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_partial(self, client: SlashSDK) -> None:
        with client.card_group.spending_constraint.with_streaming_response.update_partial(
            card_group_id="cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spending_constraint = response.parse()
            assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_partial(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            client.card_group.spending_constraint.with_raw_response.update_partial(
                card_group_id="",
            )


class TestAsyncSpendingConstraint:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_full(self, async_client: AsyncSlashSDK) -> None:
        spending_constraint = await async_client.card_group.spending_constraint.update_full(
            card_group_id="cardGroupId",
        )
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_full_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        spending_constraint = await async_client.card_group.spending_constraint.update_full(
            card_group_id="cardGroupId",
            country_rule={
                "countries": ["string"],
                "restriction": "allowlist",
            },
            merchant_category_code_rule={
                "merchant_category_codes": ["string"],
                "restriction": "allowlist",
            },
            merchant_category_rule={
                "merchant_categories": ["string"],
                "restriction": "allowlist",
            },
            merchant_rule={
                "merchants": ["string"],
                "restriction": "allowlist",
            },
            spending_rule={
                "transaction_size_limit": {
                    "maximum": {"amount_cents": 0},
                    "minimum": {"amount_cents": 0},
                },
                "utilization_limit": {
                    "limit_amount": {"amount_cents": 0},
                    "preset": "daily",
                    "start_date": "startDate",
                    "timezone": "timezone",
                },
                "utilization_limit_v2": [
                    {
                        "limit_amount": {"amount_cents": 0},
                        "preset": "daily",
                        "start_date": "startDate",
                        "timezone": "timezone",
                    }
                ],
            },
        )
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_full(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.card_group.spending_constraint.with_raw_response.update_full(
            card_group_id="cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spending_constraint = await response.parse()
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_full(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.card_group.spending_constraint.with_streaming_response.update_full(
            card_group_id="cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spending_constraint = await response.parse()
            assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_full(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            await async_client.card_group.spending_constraint.with_raw_response.update_full(
                card_group_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_partial(self, async_client: AsyncSlashSDK) -> None:
        spending_constraint = await async_client.card_group.spending_constraint.update_partial(
            card_group_id="cardGroupId",
        )
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_partial_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        spending_constraint = await async_client.card_group.spending_constraint.update_partial(
            card_group_id="cardGroupId",
            country_rule={
                "countries": ["string"],
                "restriction": "allowlist",
            },
            merchant_category_code_rule={
                "merchant_category_codes": ["string"],
                "restriction": "allowlist",
            },
            merchant_category_rule={
                "merchant_categories": ["string"],
                "restriction": "allowlist",
            },
            merchant_rule={
                "merchants": ["string"],
                "restriction": "allowlist",
            },
            spending_rule={
                "transaction_size_limit": {
                    "maximum": {"amount_cents": 0},
                    "minimum": {"amount_cents": 0},
                },
                "utilization_limit": {
                    "limit_amount": {"amount_cents": 0},
                    "preset": "daily",
                    "start_date": "startDate",
                    "timezone": "timezone",
                },
                "utilization_limit_v2": [
                    {
                        "limit_amount": {"amount_cents": 0},
                        "preset": "daily",
                        "start_date": "startDate",
                        "timezone": "timezone",
                    }
                ],
            },
        )
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_partial(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.card_group.spending_constraint.with_raw_response.update_partial(
            card_group_id="cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spending_constraint = await response.parse()
        assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_partial(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.card_group.spending_constraint.with_streaming_response.update_partial(
            card_group_id="cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spending_constraint = await response.parse()
            assert_matches_type(SpendingConstraint, spending_constraint, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_partial(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            await async_client.card_group.spending_constraint.with_raw_response.update_partial(
                card_group_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, overload

import httpx

from ...types import oauth2_get_token_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import required_args, maybe_transform, async_maybe_transform
from .userinfo import (
    UserinfoResource,
    AsyncUserinfoResource,
    UserinfoResourceWithRawResponse,
    AsyncUserinfoResourceWithRawResponse,
    UserinfoResourceWithStreamingResponse,
    AsyncUserinfoResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.oauth2_get_token_response import Oauth2GetTokenResponse

__all__ = ["Oauth2Resource", "AsyncOauth2Resource"]


class Oauth2Resource(SyncAPIResource):
    @cached_property
    def userinfo(self) -> UserinfoResource:
        return UserinfoResource(self._client)

    @cached_property
    def with_raw_response(self) -> Oauth2ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return Oauth2ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> Oauth2ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return Oauth2ResourceWithStreamingResponse(self)

    @overload
    def get_token(
        self,
        *,
        code: str,
        grant_type: Literal["authorization_code"],
        redirect_uri: str,
        code_verifier: str | Omit = omit,
        prompt: str | Omit = omit,
        scope: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Oauth2GetTokenResponse:
        """
        Get or refresh an access token

        Args:
          code_verifier: Unused by Slash, but required in the OAuth spec.

          prompt: Unused by Slash, but required in the OAuth spec.

          scope: Unused by Slash, but required in the OAuth spec.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def get_token(
        self,
        *,
        grant_type: Literal["refresh_token"],
        refresh_token: str,
        code: str | Omit = omit,
        code_verifier: str | Omit = omit,
        prompt: str | Omit = omit,
        redirect_uri: str | Omit = omit,
        scope: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Oauth2GetTokenResponse:
        """
        Get or refresh an access token

        Args:
          code: Unused by Slash, but required in the OAuth spec.

          code_verifier: Unused by Slash, but required in the OAuth spec.

          prompt: Unused by Slash, but required in the OAuth spec.

          redirect_uri: Unused by Slash, but required in the OAuth spec.

          scope: Unused by Slash, but required in the OAuth spec.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["code", "grant_type", "redirect_uri"], ["grant_type", "refresh_token"])
    def get_token(
        self,
        *,
        code: str | Omit = omit,
        grant_type: Literal["authorization_code"] | Literal["refresh_token"],
        redirect_uri: str | Omit = omit,
        code_verifier: str | Omit = omit,
        prompt: str | Omit = omit,
        scope: str | Omit = omit,
        refresh_token: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Oauth2GetTokenResponse:
        return self._post(
            "/oauth2/token",
            body=maybe_transform(
                {
                    "code": code,
                    "grant_type": grant_type,
                    "redirect_uri": redirect_uri,
                    "code_verifier": code_verifier,
                    "prompt": prompt,
                    "scope": scope,
                    "refresh_token": refresh_token,
                },
                oauth2_get_token_params.Oauth2GetTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Oauth2GetTokenResponse,
        )


class AsyncOauth2Resource(AsyncAPIResource):
    @cached_property
    def userinfo(self) -> AsyncUserinfoResource:
        return AsyncUserinfoResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOauth2ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOauth2ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOauth2ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncOauth2ResourceWithStreamingResponse(self)

    @overload
    async def get_token(
        self,
        *,
        code: str,
        grant_type: Literal["authorization_code"],
        redirect_uri: str,
        code_verifier: str | Omit = omit,
        prompt: str | Omit = omit,
        scope: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Oauth2GetTokenResponse:
        """
        Get or refresh an access token

        Args:
          code_verifier: Unused by Slash, but required in the OAuth spec.

          prompt: Unused by Slash, but required in the OAuth spec.

          scope: Unused by Slash, but required in the OAuth spec.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def get_token(
        self,
        *,
        grant_type: Literal["refresh_token"],
        refresh_token: str,
        code: str | Omit = omit,
        code_verifier: str | Omit = omit,
        prompt: str | Omit = omit,
        redirect_uri: str | Omit = omit,
        scope: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Oauth2GetTokenResponse:
        """
        Get or refresh an access token

        Args:
          code: Unused by Slash, but required in the OAuth spec.

          code_verifier: Unused by Slash, but required in the OAuth spec.

          prompt: Unused by Slash, but required in the OAuth spec.

          redirect_uri: Unused by Slash, but required in the OAuth spec.

          scope: Unused by Slash, but required in the OAuth spec.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["code", "grant_type", "redirect_uri"], ["grant_type", "refresh_token"])
    async def get_token(
        self,
        *,
        code: str | Omit = omit,
        grant_type: Literal["authorization_code"] | Literal["refresh_token"],
        redirect_uri: str | Omit = omit,
        code_verifier: str | Omit = omit,
        prompt: str | Omit = omit,
        scope: str | Omit = omit,
        refresh_token: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Oauth2GetTokenResponse:
        return await self._post(
            "/oauth2/token",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "grant_type": grant_type,
                    "redirect_uri": redirect_uri,
                    "code_verifier": code_verifier,
                    "prompt": prompt,
                    "scope": scope,
                    "refresh_token": refresh_token,
                },
                oauth2_get_token_params.Oauth2GetTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Oauth2GetTokenResponse,
        )


class Oauth2ResourceWithRawResponse:
    def __init__(self, oauth2: Oauth2Resource) -> None:
        self._oauth2 = oauth2

        self.get_token = to_raw_response_wrapper(
            oauth2.get_token,
        )

    @cached_property
    def userinfo(self) -> UserinfoResourceWithRawResponse:
        return UserinfoResourceWithRawResponse(self._oauth2.userinfo)


class AsyncOauth2ResourceWithRawResponse:
    def __init__(self, oauth2: AsyncOauth2Resource) -> None:
        self._oauth2 = oauth2

        self.get_token = async_to_raw_response_wrapper(
            oauth2.get_token,
        )

    @cached_property
    def userinfo(self) -> AsyncUserinfoResourceWithRawResponse:
        return AsyncUserinfoResourceWithRawResponse(self._oauth2.userinfo)


class Oauth2ResourceWithStreamingResponse:
    def __init__(self, oauth2: Oauth2Resource) -> None:
        self._oauth2 = oauth2

        self.get_token = to_streamed_response_wrapper(
            oauth2.get_token,
        )

    @cached_property
    def userinfo(self) -> UserinfoResourceWithStreamingResponse:
        return UserinfoResourceWithStreamingResponse(self._oauth2.userinfo)


class AsyncOauth2ResourceWithStreamingResponse:
    def __init__(self, oauth2: AsyncOauth2Resource) -> None:
        self._oauth2 = oauth2

        self.get_token = async_to_streamed_response_wrapper(
            oauth2.get_token,
        )

    @cached_property
    def userinfo(self) -> AsyncUserinfoResourceWithStreamingResponse:
        return AsyncUserinfoResourceWithStreamingResponse(self._oauth2.userinfo)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["Oauth2GetTokenParams", "Variant0", "Variant1"]


class Variant0(TypedDict, total=False):
    code: Required[str]

    grant_type: Required[Literal["authorization_code"]]

    redirect_uri: Required[str]

    code_verifier: str
    """Unused by Slash, but required in the OAuth spec."""

    prompt: str
    """Unused by Slash, but required in the OAuth spec."""

    scope: str
    """Unused by Slash, but required in the OAuth spec."""


class Variant1(TypedDict, total=False):
    grant_type: Required[Literal["refresh_token"]]

    refresh_token: Required[str]

    code: str
    """Unused by Slash, but required in the OAuth spec."""

    code_verifier: str
    """Unused by Slash, but required in the OAuth spec."""

    prompt: str
    """Unused by Slash, but required in the OAuth spec."""

    redirect_uri: str
    """Unused by Slash, but required in the OAuth spec."""

    scope: str
    """Unused by Slash, but required in the OAuth spec."""


Oauth2GetTokenParams: TypeAlias = Union[Variant0, Variant1]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import merchant_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.merchant import Merchant
from ..types.merchant_list_response import MerchantListResponse

__all__ = ["MerchantResource", "AsyncMerchantResource"]


class MerchantResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MerchantResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return MerchantResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MerchantResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return MerchantResourceWithStreamingResponse(self)

    def retrieve(
        self,
        merchant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Merchant:
        """
        Fetch details for a single merchant by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not merchant_id:
            raise ValueError(f"Expected a non-empty value for `merchant_id` but received {merchant_id!r}")
        return self._get(
            f"/merchant/{merchant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Merchant,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        search_merchant_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MerchantListResponse:
        """
        Retrieve the list of available merchant ids

        Args:
          cursor: A cursor string to fetch the next page of results

          search_merchant_name: Pass in a merchant name to filter merchants by name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/merchant",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "search_merchant_name": search_merchant_name,
                    },
                    merchant_list_params.MerchantListParams,
                ),
            ),
            cast_to=MerchantListResponse,
        )


class AsyncMerchantResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMerchantResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMerchantResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMerchantResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncMerchantResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        merchant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Merchant:
        """
        Fetch details for a single merchant by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not merchant_id:
            raise ValueError(f"Expected a non-empty value for `merchant_id` but received {merchant_id!r}")
        return await self._get(
            f"/merchant/{merchant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Merchant,
        )

    async def list(
        self,
        *,
        cursor: str | Omit = omit,
        search_merchant_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MerchantListResponse:
        """
        Retrieve the list of available merchant ids

        Args:
          cursor: A cursor string to fetch the next page of results

          search_merchant_name: Pass in a merchant name to filter merchants by name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/merchant",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "search_merchant_name": search_merchant_name,
                    },
                    merchant_list_params.MerchantListParams,
                ),
            ),
            cast_to=MerchantListResponse,
        )


class MerchantResourceWithRawResponse:
    def __init__(self, merchant: MerchantResource) -> None:
        self._merchant = merchant

        self.retrieve = to_raw_response_wrapper(
            merchant.retrieve,
        )
        self.list = to_raw_response_wrapper(
            merchant.list,
        )


class AsyncMerchantResourceWithRawResponse:
    def __init__(self, merchant: AsyncMerchantResource) -> None:
        self._merchant = merchant

        self.retrieve = async_to_raw_response_wrapper(
            merchant.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            merchant.list,
        )


class MerchantResourceWithStreamingResponse:
    def __init__(self, merchant: MerchantResource) -> None:
        self._merchant = merchant

        self.retrieve = to_streamed_response_wrapper(
            merchant.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            merchant.list,
        )


class AsyncMerchantResourceWithStreamingResponse:
    def __init__(self, merchant: AsyncMerchantResource) -> None:
        self._merchant = merchant

        self.retrieve = async_to_streamed_response_wrapper(
            merchant.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            merchant.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .developer_application_data_param import DeveloperApplicationDataParam

__all__ = ["DeveloperApplicationUpdateParams"]


class DeveloperApplicationUpdateParams(TypedDict, total=False):
    data: DeveloperApplicationDataParam

    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["DeveloperApplicationType"]

DeveloperApplicationType: TypeAlias = Literal["service"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransferCreateVirtualAccountTransferParams"]


class TransferCreateVirtualAccountTransferParams(TypedDict, total=False):
    amount_cents: Required[Annotated[float, PropertyInfo(alias="amountCents")]]
    """The amount of money to send in cents."""

    destination: Required[str]
    """The ID of the virtual account to transfer money to."""

    source: Required[str]
    """The ID of the virtual account to transfer money from.

    Can also be the virtual account linked to a primary Slash account to fund a new
    virtual account (Virtual account with the name 'Primary account').
    """

    x_idempotency_key: Required[Annotated[str, PropertyInfo(alias="X-Idempotency-Key")]]

    memo: str
    """Optional memo/description for the transfer to help differentiate transactions."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .card_group import CardGroup as CardGroup
from .spending_constraint_update_full_params import (
    SpendingConstraintUpdateFullParams as SpendingConstraintUpdateFullParams,
)
from .spending_constraint_update_partial_params import (
    SpendingConstraintUpdatePartialParams as SpendingConstraintUpdatePartialParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .loan_account import LoanAccount
from .deposit_account import DepositAccount
from .investment_account import InvestmentAccount
from .line_of_credit_account import LineOfCreditAccount

__all__ = [
    "AccountListResponse",
    "Account",
    "AccountDepositAccount",
    "AccountLoanAccount",
    "AccountLocAccount",
    "AccountInvestmentAccount",
    "AccountInsuranceAccount",
    "AccountInsuranceAccountInsuranceAccount",
    "AccountInsuranceAccountInsuranceAccountCurrency",
    "AccountAnnuityAccount",
    "AccountAnnuityAccountAnnuityAccount",
    "AccountAnnuityAccountAnnuityAccountCurrency",
    "AccountCommercialAccount",
    "AccountCommercialAccountCommercialAccount",
    "AccountCommercialAccountCommercialAccountCurrency",
    "Page",
]


class AccountDepositAccount(BaseModel):
    deposit_account: DepositAccount = FieldInfo(alias="depositAccount")
    """A deposit account. For example, a checking, savings or money market account."""


class AccountLoanAccount(BaseModel):
    loan_account: LoanAccount = FieldInfo(alias="loanAccount")
    """A loan account. For example, mortgage, student loan or auto loan."""


class AccountLocAccount(BaseModel):
    loc_account: LineOfCreditAccount = FieldInfo(alias="locAccount")
    """A line-of-credit account.

    For example, a credit card or home equity line of credit.
    """


class AccountInvestmentAccount(BaseModel):
    investment_account: InvestmentAccount = FieldInfo(alias="investmentAccount")
    """An investment account. For example, a 401K or IRA."""


class AccountInsuranceAccountInsuranceAccountCurrency(BaseModel):
    currency_code: str = FieldInfo(alias="currencyCode")
    """The ISO 4217 currency code for the currency that the account is denominated in.

    For example, "USD" for US Dollars.
    """


class AccountInsuranceAccountInsuranceAccount(BaseModel):
    """An insurance account.

    For example, whole life insurance or short-term disability.
    """

    account_id: str = FieldInfo(alias="accountId")
    """Long-term persistent identity of the account, though not an account number."""

    account_type: Literal["LONGTERMDISABILITY", "SHORTTERMDISABILITY", "UNIVERSALLIFE", "WHOLELIFE"] = FieldInfo(
        alias="accountType"
    )

    currency: AccountInsuranceAccountInsuranceAccountCurrency

    product_name: str = FieldInfo(alias="productName")
    """Marketed product name for this account.

    You can use this in UIs to assist in account selection. For example, "Slash
    Platinum".
    """

    status: Literal[
        "CLOSED", "DELINQUENT", "NEGATIVECURRENTBALANCE", "OPEN", "PAID", "PENDINGCLOSE", "PENDINGOPEN", "RESTRICTED"
    ]

    account_number_display: Optional[str] = FieldInfo(alias="accountNumberDisplay", default=None)
    """
    A masked version of the account number, which includes the last 4 digits in
    plain text.
    """

    nickname: Optional[str] = None
    """A nickname or label for the account.

    This will be unique to the account, even if the user has multiple accounts with
    the same product name. You can use this in UIs to assist in account selection.
    """


class AccountInsuranceAccount(BaseModel):
    insurance_account: AccountInsuranceAccountInsuranceAccount = FieldInfo(alias="insuranceAccount")
    """An insurance account.

    For example, whole life insurance or short-term disability.
    """


class AccountAnnuityAccountAnnuityAccountCurrency(BaseModel):
    currency_code: str = FieldInfo(alias="currencyCode")
    """The ISO 4217 currency code for the currency that the account is denominated in.

    For example, "USD" for US Dollars.
    """


class AccountAnnuityAccountAnnuityAccount(BaseModel):
    """An annuity account. For example, a fixed or variable annuity account."""

    account_id: str = FieldInfo(alias="accountId")
    """Long-term persistent identity of the account, though not an account number."""

    account_type: Literal["ANNUITY", "FIXEDANNUITY", "VARIABLEANNUITY"] = FieldInfo(alias="accountType")

    currency: AccountAnnuityAccountAnnuityAccountCurrency

    product_name: str = FieldInfo(alias="productName")
    """Marketed product name for this account.

    You can use this in UIs to assist in account selection. For example, "Slash
    Platinum".
    """

    status: Literal[
        "CLOSED", "DELINQUENT", "NEGATIVECURRENTBALANCE", "OPEN", "PAID", "PENDINGCLOSE", "PENDINGOPEN", "RESTRICTED"
    ]

    account_number_display: Optional[str] = FieldInfo(alias="accountNumberDisplay", default=None)
    """
    A masked version of the account number, which includes the last 4 digits in
    plain text.
    """

    nickname: Optional[str] = None
    """A nickname or label for the account.

    This will be unique to the account, even if the user has multiple accounts with
    the same product name. You can use this in UIs to assist in account selection.
    """


class AccountAnnuityAccount(BaseModel):
    annuity_account: AccountAnnuityAccountAnnuityAccount = FieldInfo(alias="annuityAccount")
    """An annuity account. For example, a fixed or variable annuity account."""


class AccountCommercialAccountCommercialAccountCurrency(BaseModel):
    currency_code: str = FieldInfo(alias="currencyCode")
    """The ISO 4217 currency code for the currency that the account is denominated in.

    For example, "USD" for US Dollars.
    """


class AccountCommercialAccountCommercialAccount(BaseModel):
    """A commercial account. For example, a business deposit account."""

    account_id: str = FieldInfo(alias="accountId")
    """Long-term persistent identity of the account, though not an account number."""

    account_type: Literal["COMMERCIALDEPOSIT", "COMMERCIALLOAN", "COMMERCIALLINEOFCREDIT", "COMMERCIALINVESTMENT"] = (
        FieldInfo(alias="accountType")
    )

    currency: AccountCommercialAccountCommercialAccountCurrency

    product_name: str = FieldInfo(alias="productName")
    """Marketed product name for this account.

    You can use this in UIs to assist in account selection. For example, "Slash
    Platinum".
    """

    status: Literal[
        "CLOSED", "DELINQUENT", "NEGATIVECURRENTBALANCE", "OPEN", "PAID", "PENDINGCLOSE", "PENDINGOPEN", "RESTRICTED"
    ]

    account_number_display: Optional[str] = FieldInfo(alias="accountNumberDisplay", default=None)
    """
    A masked version of the account number, which includes the last 4 digits in
    plain text.
    """

    nickname: Optional[str] = None
    """A nickname or label for the account.

    This will be unique to the account, even if the user has multiple accounts with
    the same product name. You can use this in UIs to assist in account selection.
    """


class AccountCommercialAccount(BaseModel):
    commercial_account: AccountCommercialAccountCommercialAccount = FieldInfo(alias="commercialAccount")
    """A commercial account. For example, a business deposit account."""


Account: TypeAlias = Union[
    AccountDepositAccount,
    AccountLoanAccount,
    AccountLocAccount,
    AccountInvestmentAccount,
    AccountInsuranceAccount,
    AccountAnnuityAccount,
    AccountCommercialAccount,
]


class Page(BaseModel):
    next_offset: Optional[str] = FieldInfo(alias="nextOffset", default=None)


class AccountListResponse(BaseModel):
    accounts: List[Account]

    page: Optional[Page] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .card import (
    CardResource,
    AsyncCardResource,
    CardResourceWithRawResponse,
    AsyncCardResourceWithRawResponse,
    CardResourceWithStreamingResponse,
    AsyncCardResourceWithStreamingResponse,
)
from .spending_constraint import (
    SpendingConstraintResource,
    AsyncSpendingConstraintResource,
    SpendingConstraintResourceWithRawResponse,
    AsyncSpendingConstraintResourceWithRawResponse,
    SpendingConstraintResourceWithStreamingResponse,
    AsyncSpendingConstraintResourceWithStreamingResponse,
)

__all__ = [
    "SpendingConstraintResource",
    "AsyncSpendingConstraintResource",
    "SpendingConstraintResourceWithRawResponse",
    "AsyncSpendingConstraintResourceWithRawResponse",
    "SpendingConstraintResourceWithStreamingResponse",
    "AsyncSpendingConstraintResourceWithStreamingResponse",
    "CardResource",
    "AsyncCardResource",
    "CardResourceWithRawResponse",
    "AsyncCardResourceWithRawResponse",
    "CardResourceWithStreamingResponse",
    "AsyncCardResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.oauth2.user_info import UserInfo

__all__ = ["UserinfoResource", "AsyncUserinfoResource"]


class UserinfoResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UserinfoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return UserinfoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserinfoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return UserinfoResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserInfo:
        """Get userinfo"""
        return self._get(
            "/oauth2/userinfo",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserInfo,
        )

    def submit(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserInfo:
        """Get userinfo"""
        return self._post(
            "/oauth2/userinfo",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserInfo,
        )


class AsyncUserinfoResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUserinfoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUserinfoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserinfoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncUserinfoResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserInfo:
        """Get userinfo"""
        return await self._get(
            "/oauth2/userinfo",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserInfo,
        )

    async def submit(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserInfo:
        """Get userinfo"""
        return await self._post(
            "/oauth2/userinfo",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserInfo,
        )


class UserinfoResourceWithRawResponse:
    def __init__(self, userinfo: UserinfoResource) -> None:
        self._userinfo = userinfo

        self.retrieve = to_raw_response_wrapper(
            userinfo.retrieve,
        )
        self.submit = to_raw_response_wrapper(
            userinfo.submit,
        )


class AsyncUserinfoResourceWithRawResponse:
    def __init__(self, userinfo: AsyncUserinfoResource) -> None:
        self._userinfo = userinfo

        self.retrieve = async_to_raw_response_wrapper(
            userinfo.retrieve,
        )
        self.submit = async_to_raw_response_wrapper(
            userinfo.submit,
        )


class UserinfoResourceWithStreamingResponse:
    def __init__(self, userinfo: UserinfoResource) -> None:
        self._userinfo = userinfo

        self.retrieve = to_streamed_response_wrapper(
            userinfo.retrieve,
        )
        self.submit = to_streamed_response_wrapper(
            userinfo.submit,
        )


class AsyncUserinfoResourceWithStreamingResponse:
    def __init__(self, userinfo: AsyncUserinfoResource) -> None:
        self._userinfo = userinfo

        self.retrieve = async_to_streamed_response_wrapper(
            userinfo.retrieve,
        )
        self.submit = async_to_streamed_response_wrapper(
            userinfo.submit,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types import (
    Transaction,
    TransactionListResponse,
    TransactionAggregateResponse,
    TransactionUpdateNoteResponse,
    TransactionRetrieveFeeDetailsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTransaction:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: SlashSDK) -> None:
        transaction = client.transaction.retrieve(
            "transactionId",
        )
        assert_matches_type(Transaction, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: SlashSDK) -> None:
        response = client.transaction.with_raw_response.retrieve(
            "transactionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(Transaction, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: SlashSDK) -> None:
        with client.transaction.with_streaming_response.retrieve(
            "transactionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(Transaction, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_id` but received ''"):
            client.transaction.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: SlashSDK) -> None:
        transaction = client.transaction.list()
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: SlashSDK) -> None:
        transaction = client.transaction.list(
            account_id="accountId",
            cursor="cursor",
            filter_account_id="filter:accountId",
            filter_card_id="filter:cardId",
            filter_detailed_status="pending",
            filter_from_authorized_at="filter:from_authorized_at",
            filter_from_date="filter:from_date",
            filter_legal_entity_id="filter:legalEntityId",
            filter_provider_authorization_id="filter:providerAuthorizationId",
            filter_status="pending",
            filter_to_authorized_at="filter:to_authorized_at",
            filter_to_date="filter:to_date",
            filter_virtual_account_id="filter:virtualAccountId",
        )
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: SlashSDK) -> None:
        response = client.transaction.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: SlashSDK) -> None:
        with client.transaction.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionListResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_aggregate(self, client: SlashSDK) -> None:
        transaction = client.transaction.aggregate()
        assert_matches_type(TransactionAggregateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_aggregate_with_all_params(self, client: SlashSDK) -> None:
        transaction = client.transaction.aggregate(
            account_id="accountId",
            filter_account_id="filter:accountId",
            filter_card_id="filter:cardId",
            filter_detailed_status="pending",
            filter_from_authorized_at="filter:from_authorized_at",
            filter_from_date="filter:from_date",
            filter_legal_entity_id="filter:legalEntityId",
            filter_status="pending",
            filter_to_authorized_at="filter:to_authorized_at",
            filter_to_date="filter:to_date",
            filter_virtual_account_id="filter:virtualAccountId",
        )
        assert_matches_type(TransactionAggregateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_aggregate(self, client: SlashSDK) -> None:
        response = client.transaction.with_raw_response.aggregate()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionAggregateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_aggregate(self, client: SlashSDK) -> None:
        with client.transaction.with_streaming_response.aggregate() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionAggregateResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_fee_details(self, client: SlashSDK) -> None:
        transaction = client.transaction.retrieve_fee_details(
            "transactionId",
        )
        assert_matches_type(TransactionRetrieveFeeDetailsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve_fee_details(self, client: SlashSDK) -> None:
        response = client.transaction.with_raw_response.retrieve_fee_details(
            "transactionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionRetrieveFeeDetailsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_fee_details(self, client: SlashSDK) -> None:
        with client.transaction.with_streaming_response.retrieve_fee_details(
            "transactionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionRetrieveFeeDetailsResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve_fee_details(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_id` but received ''"):
            client.transaction.with_raw_response.retrieve_fee_details(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_note(self, client: SlashSDK) -> None:
        transaction = client.transaction.update_note(
            transaction_id="transactionId",
            note="note",
        )
        assert_matches_type(TransactionUpdateNoteResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_note(self, client: SlashSDK) -> None:
        response = client.transaction.with_raw_response.update_note(
            transaction_id="transactionId",
            note="note",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionUpdateNoteResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_note(self, client: SlashSDK) -> None:
        with client.transaction.with_streaming_response.update_note(
            transaction_id="transactionId",
            note="note",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionUpdateNoteResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_note(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_id` but received ''"):
            client.transaction.with_raw_response.update_note(
                transaction_id="",
                note="note",
            )


class TestAsyncTransaction:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSlashSDK) -> None:
        transaction = await async_client.transaction.retrieve(
            "transactionId",
        )
        assert_matches_type(Transaction, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.transaction.with_raw_response.retrieve(
            "transactionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(Transaction, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.transaction.with_streaming_response.retrieve(
            "transactionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(Transaction, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_id` but received ''"):
            await async_client.transaction.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncSlashSDK) -> None:
        transaction = await async_client.transaction.list()
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        transaction = await async_client.transaction.list(
            account_id="accountId",
            cursor="cursor",
            filter_account_id="filter:accountId",
            filter_card_id="filter:cardId",
            filter_detailed_status="pending",
            filter_from_authorized_at="filter:from_authorized_at",
            filter_from_date="filter:from_date",
            filter_legal_entity_id="filter:legalEntityId",
            filter_provider_authorization_id="filter:providerAuthorizationId",
            filter_status="pending",
            filter_to_authorized_at="filter:to_authorized_at",
            filter_to_date="filter:to_date",
            filter_virtual_account_id="filter:virtualAccountId",
        )
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.transaction.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.transaction.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionListResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_aggregate(self, async_client: AsyncSlashSDK) -> None:
        transaction = await async_client.transaction.aggregate()
        assert_matches_type(TransactionAggregateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_aggregate_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        transaction = await async_client.transaction.aggregate(
            account_id="accountId",
            filter_account_id="filter:accountId",
            filter_card_id="filter:cardId",
            filter_detailed_status="pending",
            filter_from_authorized_at="filter:from_authorized_at",
            filter_from_date="filter:from_date",
            filter_legal_entity_id="filter:legalEntityId",
            filter_status="pending",
            filter_to_authorized_at="filter:to_authorized_at",
            filter_to_date="filter:to_date",
            filter_virtual_account_id="filter:virtualAccountId",
        )
        assert_matches_type(TransactionAggregateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_aggregate(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.transaction.with_raw_response.aggregate()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionAggregateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_aggregate(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.transaction.with_streaming_response.aggregate() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionAggregateResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_fee_details(self, async_client: AsyncSlashSDK) -> None:
        transaction = await async_client.transaction.retrieve_fee_details(
            "transactionId",
        )
        assert_matches_type(TransactionRetrieveFeeDetailsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_fee_details(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.transaction.with_raw_response.retrieve_fee_details(
            "transactionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionRetrieveFeeDetailsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_fee_details(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.transaction.with_streaming_response.retrieve_fee_details(
            "transactionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionRetrieveFeeDetailsResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve_fee_details(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_id` but received ''"):
            await async_client.transaction.with_raw_response.retrieve_fee_details(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_note(self, async_client: AsyncSlashSDK) -> None:
        transaction = await async_client.transaction.update_note(
            transaction_id="transactionId",
            note="note",
        )
        assert_matches_type(TransactionUpdateNoteResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_note(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.transaction.with_raw_response.update_note(
            transaction_id="transactionId",
            note="note",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionUpdateNoteResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_note(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.transaction.with_streaming_response.update_note(
            transaction_id="transactionId",
            note="note",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionUpdateNoteResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_note(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_id` but received ''"):
            await async_client.transaction.with_raw_response.update_note(
                transaction_id="",
                note="note",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..card_status import CardStatus
from .spending_constraint import SpendingConstraint

__all__ = ["Card"]


class Card(BaseModel):
    id: str

    account_id: str = FieldInfo(alias="accountId")
    """The account that this card is associated with"""

    expiry_month: str = FieldInfo(alias="expiryMonth")
    """The month the card expires formatted as MM (01, 02, ..., 12)"""

    expiry_year: str = FieldInfo(alias="expiryYear")
    """The year the card expires formatted as YYYY (2024, 2025, ...)"""

    is_physical: bool = FieldInfo(alias="isPhysical")
    """When true, a physical card has been issued. Otherwise, this is a virtual card."""

    last4: str
    """The last 4 digits of the card number"""

    name: str
    """The name assigned to the card that appears on the user dashboard"""

    status: CardStatus
    """The status of the card"""

    card_group_id: Optional[str] = FieldInfo(alias="cardGroupId", default=None)
    """The card group the card belongs to."""

    card_group_name: Optional[str] = FieldInfo(alias="cardGroupName", default=None)
    """The name of the card group the card belongs to."""

    card_product_id: Optional[str] = FieldInfo(alias="cardProductId", default=None)
    """The ID of the card product this card was created with."""

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)

    cvv: Optional[str] = None
    """
    This field will contain full CVV which will only be sent on a request for a
    single card when you set the query param "include_pan" to "true"
    """

    is_single_use: Optional[bool] = FieldInfo(alias="isSingleUse", default=None)
    """
    When true, the card will be automatically closed after a single authorization
    attempt. Note that the card will be closed even if the authorization declines or
    drops
    """

    pan: Optional[str] = None
    """
    This field contains the full PAN which will only be sent on a request for a
    single card when you set the query param "include_pan" to "true"
    """

    spending_constraint: Optional[SpendingConstraint] = FieldInfo(alias="spendingConstraint", default=None)
    """The spending constraint applied to the card"""

    user_data: Optional[object] = FieldInfo(alias="userData", default=None)
    """Arbitrary information that can be attached to the card.

    See the [`PATCH /card/{cardId}`](api-reference/card-patch) endpoint for more
    details on how to add user data.
    """

    virtual_account_id: Optional[str] = FieldInfo(alias="virtualAccountId", default=None)
    """The virtual account that this card is associated with"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AccountRetrieveContactResponse", "Address", "Holder", "HolderName", "Telephone"]


class Address(BaseModel):
    city: str

    country: str
    """Iso3166CountryCode"""

    line1: str

    line2: str

    line3: Optional[str] = None

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)

    region: Optional[str] = None
    """State or province or territory."""


class HolderName(BaseModel):
    first: str

    last: str

    middle: str

    prefix: Optional[str] = None

    suffix: Optional[str] = None


class Holder(BaseModel):
    name: HolderName


class Telephone(BaseModel):
    number: str
    """Telephone subscriber number defined by ITU-T recommendation E.164"""

    type: Literal["HOME", "BUSINESS", "CELL", "FAX"]

    country: Optional[str] = None
    """Country calling codes defined by ITU-T recommendations E.123 and E.164"""


class AccountRetrieveContactResponse(BaseModel):
    addresses: List[Address]

    emails: List[str]

    holders: List[Holder]

    telephones: List[Telephone]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .fdx import (
    FdxResource,
    AsyncFdxResource,
    FdxResourceWithRawResponse,
    AsyncFdxResourceWithRawResponse,
    FdxResourceWithStreamingResponse,
    AsyncFdxResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)

__all__ = [
    "CustomersResource",
    "AsyncCustomersResource",
    "CustomersResourceWithRawResponse",
    "AsyncCustomersResourceWithRawResponse",
    "CustomersResourceWithStreamingResponse",
    "AsyncCustomersResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "FdxResource",
    "AsyncFdxResource",
    "FdxResourceWithRawResponse",
    "AsyncFdxResourceWithRawResponse",
    "FdxResourceWithStreamingResponse",
    "AsyncFdxResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["Oauth2GetTokenResponse"]


class Oauth2GetTokenResponse(BaseModel):
    access_token: str

    expires_in: int

    id_token: str

    refresh_token: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Money"]


class Money(BaseModel):
    """Represents a monetary value"""

    amount_cents: int = FieldInfo(alias="amountCents")
    """The amount in cents"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .card.spending_constraint_param import SpendingConstraintParam

__all__ = ["CardCreateParams"]


class CardCreateParams(TypedDict, total=False):
    name: Required[str]

    type: Required[Literal["virtual"]]
    """Specify the type of card you'd like to create.

    At the moment, only virtual cards are supported.
    """

    account_id: Annotated[str, PropertyInfo(alias="accountId")]
    """The ID of the account to create the card under.

    You can get this by calling `GET /account`. This field is required unless you
    are authenticating via API key, in which case it will default to your first
    commercial account. We recommend supplying this even if you are authenticating
    via API key.
    """

    card_group_id: Annotated[str, PropertyInfo(alias="cardGroupId")]

    card_product_id: Annotated[str, PropertyInfo(alias="cardProductId")]
    """
    The ID of the card product to use when creating this card, if not specified a
    random card product will be chosen.
    """

    is_single_use: Annotated[bool, PropertyInfo(alias="isSingleUse")]
    """Defaults to false.

    When set to true, the card will be automatically closed after a single
    authorization attempt. Note that the card will be closed even if the
    authorization declines or drops
    """

    spending_constraint: Annotated[SpendingConstraintParam, PropertyInfo(alias="spendingConstraint")]
    """A constraint that can be applied to a CardGroupSpendingRule"""

    user_data: Annotated[Dict[str, object], PropertyInfo(alias="userData")]
    """Arbitrary information that can be attached to the card.

    This should be a JSON object and cannot exceed 4kb.
    """

    virtual_account_id: Annotated[str, PropertyInfo(alias="virtualAccountId")]
    """The ID of the virtual account to create the card under.

    Virtual accounts can be retrieved by calling `GET /virtual-account`.
    """
